package com.supwisdom.institute.backend.system.domain.repo;

import java.util.Map;
import java.util.Optional;

import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Repository;

import com.supwisdom.institute.backend.common.framework.repo.BaseJpaRepository;
import com.supwisdom.institute.backend.common.util.MapBeanUtils;
import com.supwisdom.institute.backend.system.domain.entity.Account;

@Repository
public interface AccountRepository extends BaseJpaRepository<Account> {
  
//  public default Page<Account> selectPageList(int pageIndex, int pageSize, Account probe) {
//    
//    ExampleMatcher matcher = ExampleMatcher.matching()
//        .withMatcher("username", ExampleMatcher.GenericPropertyMatchers.contains())
//        .withMatcher("name", ExampleMatcher.GenericPropertyMatchers.contains())
//        .withMatcher("status", ExampleMatcher.GenericPropertyMatchers.exact());
//    
//    PageRequest pageRequest = PageRequest.of(pageIndex, pageSize);
//    Example<Account> example = Example.of(probe, matcher);
//    
//    Page<Account> page = this.findAll(example, pageRequest);
//    
//    return page;
//  }
  
  @Override
  public default Page<Account> selectPageList(boolean loadAll, int pageIndex, int pageSize, Map<String, Object> mapBean, Map<String, String> orderBy) {
    if (loadAll) {
      pageIndex = 0;
      pageSize = Integer.MAX_VALUE;
    }
    
    Account probe = new Account();
    if (mapBean != null) {
      probe.setDeleted(MapBeanUtils.getBoolean(mapBean, "deleted"));
      probe.setUsername(MapBeanUtils.getString(mapBean, "username"));
      probe.setName(MapBeanUtils.getString(mapBean, "name"));
      probe.setStatus(MapBeanUtils.getString(mapBean, "status"));
    }
    
    ExampleMatcher matcher = ExampleMatcher.matching()
        .withMatcher("deleted", ExampleMatcher.GenericPropertyMatchers.exact())
        .withMatcher("username", ExampleMatcher.GenericPropertyMatchers.contains())
        .withMatcher("name", ExampleMatcher.GenericPropertyMatchers.contains())
        .withMatcher("status", ExampleMatcher.GenericPropertyMatchers.exact());
    
    PageRequest pageRequest = PageRequest.of(pageIndex, pageSize);
    Example<Account> example = Example.of(probe, matcher);
    
    Page<Account> page = this.findAll(example, pageRequest);
    
    return page;
  }
  
  /*
  public default User selectById(String id) {
    
    try {
      Optional<User> entity = this.findById(id);
      
      return entity.get();
    } catch(RuntimeException e) {
      System.out.println("RuntimeException:"+e.getMessage());
    } catch(Exception e) {
      System.out.println("Exception:"+e.getMessage());
    }
    
    return null;
  }
  
  public default User insert(User entity) {
    
    if (entity.getCompanyId() == null || entity.getCompanyId().isEmpty()) {
      entity.setCompanyId("1");
    }
    
    entity.setDeleted(false);
    //entity.setAddAccount(AuthUtil.getRemoteUser());
    entity.setAddTime(Calendar.getInstance().getTime());
    
    User e = this.save(entity);
    
    return e;
  }
  
  public default User update(User entity) {
    
    //entity.setEditAccount(AuthUtil.getRemoteUser());
    entity.setEditTime(Calendar.getInstance().getTime());
    
    User e = this.save(entity);
    
    return e;
  }
  */
  
  public default Account selectByUsername(String username) {
    Account probe = new Account();
    probe.setUsername(username);
    
    ExampleMatcher matcher = ExampleMatcher.matching()
        .withMatcher("username", ExampleMatcher.GenericPropertyMatchers.exact());
    
    Example<Account> example = Example.of(probe, matcher);
    
    Optional<Account> u = this.findOne(example);
    
    if (u.isPresent()) {
      return u.get();
    }
    
    return null;
  }

}
